"""
It is a launcher for starting subprocesses for server and clients of two types: senders and listeners.
for more information:
https://stackoverflow.com/questions/67348716/kill-process-do-not-kill-the-subprocess-and-do-not-close-a-terminal-window
"""

import os
import signal
import subprocess
import sys
from time import sleep

from messeghost8740 import server
from messenger8740 import client


PYTHON_PATH = sys.executable
BASE_PATH = os.path.dirname(os.path.abspath(__file__))


def get_subprocess(file_with_args):
    sleep(0.2)
    file_full_path = f"{PYTHON_PATH} {BASE_PATH}/{file_with_args}"
    args = ["gnome-terminal", "--disable-factory", "--", "bash", "-c", file_full_path]
    return subprocess.Popen(args, preexec_fn=os.setpgrp)


if __name__ == "__main__":
    process = []
    while True:
        TEXT_FOR_INPUT = "Выберите действие: q - выход, s - запустить сервер и клиенты, x - закрыть все окна: "
        action = input(TEXT_FOR_INPUT)

        if action == "q":
            break
        elif action == "s":
            process.append(get_subprocess("server.py"))

            for i in range(2):
                process.append(get_subprocess(f"client.py -n test{i+1}"))

        elif action == "x":
            while process:
                victim = process.pop()
                os.killpg(victim.pid, signal.SIGINT)



from sqlalchemy.orm import sessionmaker
import datetime
import hashlib
import binascii

from db_server import metadata, engine
from db_server import Users, ActiveUsers, UsersHistory, LoginHistory, Contacts


class Storage:
    def __init__(self, engine=engine):
        metadata.create_all(engine)  # Инициализация базы данных и создание таблиц
        self.session = sessionmaker(bind=engine)()  # Создание сессии

    def check_user_existing(self, username: str) -> bool:
        # Проверяет, существует ли пользователь с данным именем username.
        # Если пользователь существует, query.first() вернет не None, а фукнция вернет True
        # В случае ошибки запроса, считаем что пользователь не найден, вернет False

        try:
            user = self.session.query(Users).filter_by(name=username).first()
            return user is not None
        except:
            return False

    def get_hash(self, username: str):
        hash_string = self.session.query(Users).filter_by(name=username).first().password_hash
        return hash_string

    def add_user(self, username, password, last_login=None) -> Users:
        # Добавляет нового пользователя в базу данных.
        # Проверяем, существует ли уже такой пользователь
        # Если нет, то добавляем нового пользователя
        # Устанавливаем время последнего входа
        try:
            if not self.check_user_existing(username):
                # Создание пользователя
                # Создание хеша пароля. В качестве соли будем использовать логин в  нижнем регистре.
                passwd_bytes = password.encode("utf-8")
                salt = username.lower().encode("utf-8")
                hash_bytes = hashlib.pbkdf2_hmac("sha512", passwd_bytes, salt, 10000)
                hash_string = binascii.hexlify(hash_bytes)
                new_user = Users(username, password_hash=hash_string)
                new_user.last_login = last_login if last_login else datetime.datetime.now()
                self.session.add(new_user)
                self.session.commit()
                return new_user

        except Exception as e:
            print(f"Ошибка добавления пользователя {username}: {e}")
            self.session.rollback()  # Откатываем изменения в случае ошибки
            return None  # В случае ошибки

    def remove_user(self, username) -> None:
        self.session.query(Users).filter_by(name=username).delete()
        self.session.commit()

    def user_login(self, username, ip_address, port, pubkey="") -> None:
        # Функция выполняющаяся при входе пользователя, записывает в базу факт входа
        # Запрос в таблицу пользователей на наличие там пользователя с таким именем
        # Если имя пользователя уже присутствует в таблице, обновляем время последнего входа
        # и проверяем корректность ключа. Если клиент прислал новый ключ,
        # сохраняем его.
        # Если имя пользователя уже присутствует в таблице, обновляем время последнего входа
        # Если нет, то создаём нового пользователя
        # Теперь можно создать запись в таблицу активных пользователей о факте входа.
        # и сохранить в историю входов

        user = self.session.query(Users).filter_by(name=username).first()
        user.last_login = datetime.datetime.now()
        if user.pubkey != pubkey:
            user.pubkey = pubkey

        new_active_user = ActiveUsers(user.id, ip_address, port, datetime.datetime.now())
        self.session.add(new_active_user)

        user_history = UsersHistory(user.id)
        self.session.add(user_history)

        logon_history = LoginHistory(user.id, datetime.datetime.now(), ip_address, port)
        self.session.add(logon_history)

        self.session.commit()

    def user_logout(self, username: str) -> None:
        # Запрашиваем пользователя, что покидает нас.
        # Удаляем его из таблицы активных пользователей.

        user = self.session.query(Users).filter_by(name=username).first()
        self.session.query(ActiveUsers).filter_by(user=user.id).delete()
        self.session.commit()

    def get_pubkey(self, name):
        """Метод получения публичного ключа пользователя."""
        user = self.session.query(Users).filter_by(name=name).first()
        return user.pubkey

    def process_message(self, sender_name: str, recipient_name: str) -> None:
        # Функция фиксирует передачу сообщения и делает соответствующие отметки в БД
        # Получаем ID отправителя и получателя
        # Запрашиваем строки из истории и увеличиваем счётчики

        sender_id = self.session.query(Users).filter_by(name=sender_name).first().id
        sender_history = self.session.query(UsersHistory).filter_by(user=sender_id).first()
        sender_history.sent += 1

        recipient_id = self.session.query(Users).filter_by(name=recipient_name).first().id
        recipient_history = self.session.query(UsersHistory).filter_by(user=recipient_id).first()
        recipient_history.accepted += 1

        self.session.commit()

    def add_contact(self, user, contact):
        # Функция добавляет контакт для пользователя.
        # Получаем ID пользователей
        # Проверяем что не дубль и что контакт может существовать (полю пользователь мы доверяем)
        # Создаём объект и заносим его в базу

        user = self.session.query(Users).filter_by(name=user).first()
        contact = self.session.query(Users).filter_by(name=contact).first()

        if not contact or self.session.query(Contacts).filter_by(user=user.id, contact=contact.id).count():
            return

        contact_row = Contacts(user.id, contact.id)
        self.session.add(contact_row)
        self.session.commit()

    def remove_contact(self, user, contact):
        # Функция удаляет контакт из базы данных
        # Получаем ID пользователей
        # Проверяем что контакт может существовать (полю пользователь мы доверяем)
        # Удаляем требуемое

        user = self.session.query(Users).filter_by(name=user).first()
        contact = self.session.query(Users).filter_by(name=contact).first()

        if not contact:
            return

        self.session.query(Contacts).filter(Contacts.user == user.id, Contacts.contact == contact.id).delete()
        self.session.commit()

    def users_list(self):
        # Функция возвращает список известных пользователей со временем последнего входа.
        # Запрос строк таблицы пользователей.
        # Возвращаем список кортежей

        query = self.session.query(Users.name, Users.last_login)
        return query.all()

    def active_users_list(self):
        # Функция возвращает список активных пользователей
        # Запрашиваем соединение таблиц и собираем кортежи имя, адрес, порт, время.
        # Возвращаем список кортежей

        query = self.session.query(
            Users.name,
            ActiveUsers.ip_address,
            ActiveUsers.port,
            ActiveUsers.login_time,
        ).join(Users)
        return query.all()

    def login_history(self, username=None):
        # Функция возвращающая историю входов по пользователю или всем пользователям
        # Запрашиваем историю входа
        # Если было указано имя пользователя, то фильтруем по нему
        # Возвращаем список кортежей

        query = self.session.query(
            Users.name,
            LoginHistory.date_time,
            LoginHistory.ip,
            LoginHistory.port,
        ).join(Users)

        if username:
            query = query.filter(Users.name == username)

        return query.all()

    def get_contacts(self, username):
        # Функция возвращает список контактов пользователя.
        # Запрашиваем указанного пользователя
        # Запрашиваем его список контактов
        # выбираем только имена пользователей и возвращаем их.

        user = self.session.query(Users).filter_by(name=username).one()
        contacts = (
            self.session.query(Contacts, Users.name)
            .filter_by(user=user.id)
            .join(Users, Contacts.contact == Users.id)
            .all()
        )
        return [contact[1] for contact in contacts]

    def message_history(self):
        # Функция возвращает количество переданных и полученных сообщений
        # Возвращаем список кортежей

        query = self.session.query(
            Users.name,
            Users.last_login,
            UsersHistory.sent,
            UsersHistory.accepted,
        ).join(Users)

        return query.all()



from sqlalchemy import Table, Column, Integer, String, Text, DateTime
from .database import metadata, registry
import datetime


# Таблицы
users = Table(
    "known_users",
    metadata,
    Column("id", Integer, primary_key=True),
    Column("username", String),
)


history = Table(
    "message_history",
    metadata,
    Column("id", Integer, primary_key=True),
    Column("from_user", String),
    Column("to_user", String),
    Column("message", Text),
    Column("date", DateTime),
)


contacts = Table(
    "contacts",
    metadata,
    Column("id", Integer, primary_key=True),
    Column("name", String, unique=True),
)


# Классы
class KnownUsers:
    def __init__(self, user):
        self.id = None
        self.username = user


class MessageHistory:
    def __init__(self, from_user, to_user, message):
        self.id = None
        self.from_user = from_user
        self.to_user = to_user
        self.message = message
        self.date = datetime.datetime.now()


class Contacts:
    def __init__(self, contact):
        self.id = None
        self.name = contact


# Регистрация классов
registry.map_imperatively(KnownUsers, users)
registry.map_imperatively(MessageHistory, history)
registry.map_imperatively(Contacts, contacts)



"""Утилиты"""

import json
import sys

sys.path.append("../")
# from common.decos import log
from logs.logger import log

from core.variables import *


# Утилита приёма и декодирования сообщения
# принимает байты выдаёт словарь, если приняточто-то другое отдаёт ошибку типа
@log
def get_message(client):
    """Summary line.

    Extended description of function.

    Args:
        arg1 (int): Description of arg1
        arg2 (str): Description of arg2

    Returns:
        bool: Description of return value

    """
    encoded_response = client.recv(MAX_PACKAGE_LENGTH)
    json_response = encoded_response.decode(ENCODING)
    response = json.loads(json_response)
    if isinstance(response, dict):
        return response
    else:
        raise TypeError


# Утилита кодирования и отправки сообщения
# принимает словарь и отправляет его
@log
def send_message(sock, message):
    js_message = json.dumps(message)
    encoded_message = js_message.encode(ENCODING)
    sock.send(encoded_message)



import dis


# Метакласс для проверки соответствия сервера:
class ServerMaker(type):
    def __init__(cls, clsname, bases, clsdict):
        # Список методов, которые используются в функциях класса:
        methods = []
        # Атрибуты, вызываемые функциями классов
        attrs = []
        for func in clsdict:
            # Пробуем
            try:
                ret = dis.get_instructions(clsdict[func])
                # Если не функция то ловим исключение
            except TypeError:
                pass
            else:
                # Раз функция разбираем код, получая используемые методы и атрибуты.
                for i in ret:
                    if i.opname == "LOAD_GLOBAL":
                        if i.argval not in methods:
                            methods.append(i.argval)
                    elif i.opname == "LOAD_ATTR":
                        if i.argval not in attrs:
                            attrs.append(i.argval)
        # Если обнаружено использование недопустимого метода connect, бросаем исключение:
        if "connect" in methods:
            raise TypeError("Использование метода connect недопустимо в серверном классе")
        # Если сокет не инициализировался константами SOCK_STREAM(TCP) AF_INET(IPv4), тоже исключение.
        if not ("SOCK_STREAM" in attrs and "AF_INET" in attrs):
            raise TypeError("Некорректная инициализация сокета.")
        super().__init__(clsname, bases, clsdict)



import pytest
import datetime

from db_server import Users, UsersHistory


@pytest.mark.only
def test_user_login(storage, session):
    username = "test_user"
    password = "pass"
    ip_address = "127.0.0.1"
    port = 1234

    new_user = storage.add_user(username, password)  # Сначала добавляем пользователя
    print(new_user)
    storage.user_login(username=username, ip_address=ip_address, port=port)
    user = session.query(Users).filter_by(name=username).first()
    assert user is not None  # Пользователь должен существовать
    assert user.name == username  # Имя должно соответствовать


def test_user_logout(storage, session):
    username = "user_for_logout"
    storage.add_user(username, password="pass")  # Сначала добавляем пользователя
    storage.user_login(username, "127.0.0.1", 1234)  # Пользователь входит
    storage.user_logout(username)  # Пользователь выходит

    # Проверяем, что пользователь больше не в списке активных пользователей
    active_users = storage.active_users_list()
    assert username not in [user[0] for user in active_users]


def test_add_user(storage, session):
    username = "user_to_add"
    new_user = storage.add_user(username, password="pass")

    # Проверяем, что пользователь действительно добавлен в базу
    assert new_user.id == session.query(Users).filter_by(name=username).first().id
    assert storage.check_user_existing(username) is True


def test_check_user_existing(storage, session):
    existing_user = "existing_user"
    non_existing_user = "non_existing_user"
    storage.add_user(existing_user, password="some_hash")

    # Проверяем существующего и несуществующего пользователя
    assert storage.check_user_existing(existing_user) is True
    assert storage.check_user_existing(non_existing_user) is False


def test_add_contact(storage, session):
    user1, user2 = "user1", "user2"
    storage.add_user(user1, password="some_hash")
    storage.add_user(user2, password="some_hash")
    storage.add_contact(user1, user2)

    contacts = storage.get_contacts(user1)
    assert user2 in contacts  # Проверяем, что user2 теперь контакт user1


def test_remove_contact(storage, session):
    user1, user2 = "user1", "user2"
    storage.add_user(user1, password="some_hash")
    storage.add_user(user2, password="some_hash")
    storage.add_contact(user1, user2)
    storage.remove_contact(user1, user2)

    contacts = storage.get_contacts(user1)
    assert user2 not in contacts  # Проверяем, что user2 удален из контактов user1


def test_users_list(storage, session):
    username = "new_user"
    storage.add_user(username, password="pass")
    users = storage.users_list()

    # Проверяем, что новый пользователь появился в списке пользователей
    assert username in [user[0] for user in users]


def test_active_users_list(storage, session):
    username = "active_user"
    storage.add_user(username, password="pass")
    storage.user_login(username, "127.0.0.1", 1234)
    active_users = storage.active_users_list()

    # Проверяем, что пользователь теперь активный
    assert username in [user[0] for user in active_users]


def test_login_history(storage, session):
    username = "user_with_history"
    storage.add_user(username, password="pass")
    storage.user_login(username, "127.0.0.1", 1234)

    history = storage.login_history(username)
    # Проверяем, что история входа содержит запись о входе
    assert any(h[0] == username for h in history)


def test_get_contacts(storage, session):
    user1, user2 = "contact_user1", "contact_user2"
    storage.add_user(user1, password="some_hash")
    storage.add_user(user2, password="some_hash")
    storage.add_contact(user1, user2)
    contacts = storage.get_contacts(user1)

    # Проверяем, что user2 в списке контактов user1
    assert user2 in contacts


def test_process_message(storage, session):
    # Добавляем пользователей и имитируем их вход
    sender_name, recipient_name = "user1", "user2"
    sender = storage.add_user(sender_name, password="some_hash")
    storage.user_login(sender_name, "127.0.0.1", 1234)
    recipient = storage.add_user(recipient_name, password="some_hash")
    storage.user_login(recipient_name, "127.0.0.1", 1235)

    # Проверяем, что пользователи существует
    assert storage.check_user_existing(sender_name) is True
    assert storage.check_user_existing(recipient_name) is True
    assert sender is not None
    assert recipient is not None

    # Проверка, что истории для пользователей существуют
    sender_history = session.query(UsersHistory).filter_by(user=sender.id).first()
    recipient_history = session.query(UsersHistory).filter_by(user=recipient.id).first()
    assert sender_history is not None  # Проверяем, что история отправителя существует
    assert sender_history.sent == 0
    assert recipient_history is not None  # Проверяем, что история получателя существует
    assert recipient_history.accepted == 0

    # Проверяем историю сообщений после отправки сообщения
    storage.process_message(sender_name, recipient_name)
    updated_sender_history = session.query(UsersHistory).filter_by(user=sender.id).first()
    updated_recipient_history = session.query(UsersHistory).filter_by(user=recipient.id).first()
    session.refresh(sender_history)  # Обновление конкретных объектов
    session.refresh(recipient_history)  # Обновление конкретных объектов
    assert updated_sender_history.sent == 1
    assert updated_recipient_history.accepted == 1


def test_message_history(storage, session):
    sender, recipient = "msg_user1", "msg_user2"
    storage.add_user(sender, password="some_hash")
    storage.user_login(sender, "127.0.0.1", 1234)
    storage.add_user(recipient, password="some_hash")
    storage.user_login(recipient, "127.0.0.1", 1235)

    storage.process_message(sender, recipient)
    messages = storage.message_history()
    # Проверяем историю сообщений для учета переданных и полученных сообщений
    assert any(m[0] == sender and m[2] == 1 for m in messages)
    assert any(m[0] == recipient and m[3] == 1 for m in messages)


if __name__ == "__main__":
    pytest.main()



from sqlalchemy import create_engine, MetaData
from sqlalchemy.orm import registry

engine = create_engine(
    "sqlite:///db/server_db.db3",
    echo=False,
    pool_recycle=7200,
    connect_args={"check_same_thread": False},
)


metadata = MetaData()  # создаем экземпляр MetaData
registry = registry()  # создаем экземпляр registry


# db_session = sessionmaker(bind=engine)()
# todo: from source engine
# engine = create_engine(
#     f"sqlite:///{path}",
#     echo=False,
#     pool_recycle=7200,
#     connect_args={"check_same_thread": False},
# )


# Варианты путей к БД
# "sqlite:///db_server.db3",
# f"sqlite:///{path}",
# "db_test_server.db3"
# os.path.join(config["SETTINGS"]["Database_path"], config["SETTINGS"]["Database_file"])



# models.py
from sqlalchemy import Table, Column, Integer, String, DateTime, ForeignKey, Text
from .database import metadata, registry
import datetime

# Таблицы
users_table = Table(
    "Users",
    metadata,
    Column("id", Integer, primary_key=True),
    Column("name", String, unique=True),
    Column("password_hash", String),
    Column("last_login", DateTime),
    Column("pubkey", Text),
)


active_users_table = Table(
    "Active_users",
    metadata,
    Column("id", Integer, primary_key=True),
    Column("user", ForeignKey("Users.id"), unique=True),
    Column("ip_address", String),
    Column("port", Integer),
    Column("login_time", DateTime),
)

user_login_history = Table(
    "Login_history",
    metadata,
    Column("id", Integer, primary_key=True),
    Column("name", ForeignKey("Users.id")),
    Column("date_time", DateTime),
    Column("ip", String),
    Column("port", String),
)

contacts = Table(
    "Contacts",
    metadata,
    Column("id", Integer, primary_key=True),
    Column("user", ForeignKey("Users.id")),
    Column("contact", ForeignKey("Users.id")),
)

users_history_table = Table(
    "History",
    metadata,
    Column("id", Integer, primary_key=True),
    Column("user", ForeignKey("Users.id")),
    Column("sent", Integer),
    Column("accepted", Integer),
)


# Классы
class Users:
    def __init__(self, username, password_hash):
        self.id = None
        self.name = username
        self.password_hash = password_hash
        self.last_login = datetime.datetime.now()
        self.pubkey = None


class ActiveUsers:
    def __init__(self, user_id, ip_address, port, login_time):
        self.id = None
        self.user = user_id
        self.ip_address = ip_address
        self.port = port
        self.login_time = login_time


class LoginHistory:
    def __init__(self, name, date, ip, port):
        self.id = None
        self.name = name
        self.date_time = date
        self.ip = ip
        self.port = port


class Contacts:
    def __init__(self, user, contact):
        self.id = None
        self.user = user
        self.contact = contact


class UsersHistory:
    def __init__(self, user):
        self.id = None
        self.user = user
        self.sent = 0
        self.accepted = 0


# Регистрация классов
registry.map_imperatively(Users, users_table)
registry.map_imperatively(ActiveUsers, active_users_table)
registry.map_imperatively(LoginHistory, user_login_history)
registry.map_imperatively(Contacts, contacts)
registry.map_imperatively(UsersHistory, users_history_table)



from PyQt5.QtWidgets import QDialog, QPushButton, QTableView
from PyQt5.QtGui import QStandardItemModel, QStandardItem
from PyQt5.QtCore import Qt


class StatWindow(QDialog):
    """
    Класс - окно со статистикой пользователей
    """

    def __init__(self, database):
        super().__init__()

        self.database = database
        self.initUI()

    def initUI(self):
        # Настройки окна:
        self.setWindowTitle("Статистика клиентов")
        self.setFixedSize(600, 700)
        self.setAttribute(Qt.WA_DeleteOnClose)

        # Кнапка закрытия окна
        self.close_button = QPushButton("Закрыть", self)
        self.close_button.move(250, 650)
        self.close_button.clicked.connect(self.close)

        # Лист с собственно статистикой
        self.stat_table = QTableView(self)
        self.stat_table.move(10, 10)
        self.stat_table.setFixedSize(580, 620)

        self.create_stat_model()

    def create_stat_model(self):
        """Метод реализующий заполнение таблицы статистикой сообщений."""
        # Список записей из базы
        stat_list = self.database.message_history()

        # Объект модели данных:
        list = QStandardItemModel()
        list.setHorizontalHeaderLabels(
            ["Имя Клиента", "Последний раз входил", "Сообщений отправлено", "Сообщений получено"]
        )
        for row in stat_list:
            user, last_seen, sent, recvd = row
            user = QStandardItem(user)
            user.setEditable(False)
            last_seen = QStandardItem(str(last_seen.replace(microsecond=0)))
            last_seen.setEditable(False)
            sent = QStandardItem(str(sent))
            sent.setEditable(False)
            recvd = QStandardItem(str(recvd))
            recvd.setEditable(False)
            list.appendRow([user, last_seen, sent, recvd])
        self.stat_table.setModel(list)
        self.stat_table.resizeColumnsToContents()
        self.stat_table.resizeRowsToContents()



import socket


class Port:
    """
    Класс - дескриптор для номера порта.
    Позволяет использовать только порты с 1023 по 65536.
    При попытке установить неподходящий номер порта генерирует исключение.
    """

    def __set__(self, instance, value):
        if not 1023 < value < 65536:
            # logger.critical(
            #     f"Попытка запуска с указанием неподходящего порта {value}. Допустимы адреса с 1024 до 65535."
            # )
            raise TypeError("Некорректрый номер порта")
        instance.__dict__[self.name] = value

    def __set_name__(self, owner, name):
        self.name = name


def login_required(func):
    """
    Декоратор, проверяющий, что клиент авторизован на сервере.
    Проверяет, что передаваемый объект сокета находится в
    списке авторизованных клиентов.
    За исключением передачи словаря-запроса
    на авторизацию. Если клиент не авторизован,
    генерирует исключение TypeError
    """

    def checker(*args, **kwargs):
        # проверяем, что первый аргумент - экземпляр MessageProcessor
        # Импортить необходимо тут, иначе ошибка рекурсивного импорта.
        from core.core import MessageProcessor
        from core.variables import ACTION, PRESENCE

        if isinstance(args[0], MessageProcessor):
            found = False
            for arg in args:
                if isinstance(arg, socket.socket):
                    # Проверяем, что данный сокет есть в списке names класса
                    # MessageProcessor
                    for client in args[0].names:
                        if args[0].names[client] == arg:
                            found = True

            # Теперь надо проверить, что передаваемые аргументы не presence
            # сообщение. Если presence, то разрешаем
            for arg in args:
                if isinstance(arg, dict):
                    if ACTION in arg and arg[ACTION] == PRESENCE:
                        found = True
            # Если не не авторизован и не сообщение начала авторизации, то
            # вызываем исключение.
            if not found:
                raise TypeError
        return func(*args, **kwargs)

    return checker



import sys
import os
import argparse

import configparser

from core.core import MessageProcessor
from core.utils import *
from core.variables import *
from db_server import Storage

from PyQt5.QtWidgets import QApplication
from PyQt5.QtCore import Qt
from gui.main_window import MainWindow

from logs.logger import logger, log


@log
def arg_parser(default_port, default_address):
    """Парсер аргументов коммандной строки."""
    logger.debug(f"Инициализация парсера аргументов коммандной строки: {sys.argv}")
    parser = argparse.ArgumentParser()
    parser.add_argument("-p", default=default_port, type=int, nargs="?")
    parser.add_argument("-a", default=default_address, nargs="?")
    parser.add_argument("--no_gui", action="store_true")
    namespace = parser.parse_args(sys.argv[1:])
    listen_address = namespace.a
    listen_port = namespace.p
    gui_flag = namespace.no_gui
    logger.debug("Аргументы успешно загружены.")
    return listen_address, listen_port, gui_flag


@log
def config_load():
    """Парсер конфигурационного ini файла."""
    config = configparser.ConfigParser()
    dir_path = os.path.dirname(os.path.realpath(__file__))
    config.read(f"{dir_path}/{'server.ini'}")
    # Если конфиг файл загружен правильно, запускаемся, иначе конфиг по
    # умолчанию.
    if "SETTINGS" in config:
        return config
    else:
        config.add_section("SETTINGS")
        config.set("SETTINGS", "Default_port", str(DEFAULT_PORT))
        config.set("SETTINGS", "Listen_Address", "")
        config.set("SETTINGS", "Database_path", "")
        config.set("SETTINGS", "Database_file", "server_database.db3")
        return config


@log
def main():
    """Основная функция"""
    # Загрузка файла конфигурации сервера
    config = config_load()

    # Загрузка параметров командной строки, если нет параметров, то задаём
    # значения по умоланию.
    listen_address, listen_port, gui_flag = arg_parser(
        config["SETTINGS"]["Default_port"], config["SETTINGS"]["Listen_Address"]
    )

    # Инициализация базы данных
    db = Storage()

    # Создание экземпляра класса - сервера и его запуск:
    server = MessageProcessor(listen_address, listen_port, db)
    server.daemon = True
    server.start()

    # Если  указан параметр без GUI то запускаем простенький обработчик
    # консольного ввода
    if gui_flag:
        while True:
            command = input("Введите exit для завершения работы сервера.")
            if command == "exit":
                # Если выход, то завршаем основной цикл сервера.
                server.running = False
                server.join()
                break

    # Если не указан запуск без GUI, то запускаем GUI:
    else:
        # Создаём графическое окуружение для сервера:
        server_app = QApplication(sys.argv)
        server_app.setAttribute(Qt.AA_DisableWindowContextHelpButton)
        main_window = MainWindow(db, server, config)

        # Запускаем GUI
        server_app.exec_()

        # По закрытию окон останавливаем обработчик сообщений
        server.running = False


if __name__ == "__main__":
    main()



import argparse
import sys
from pathlib import Path
import os

from core.core import ClientTransport
from core.variables import *
from core.errors_user import ServerError
from db_client import Storage

from PyQt5.QtWidgets import QApplication, QMessageBox


from logs.logger import logger, log
from Cryptodome.PublicKey import RSA


# Парсер аргументов коммандной строки
@log
def arg_parser():
    """
    Парсер аргументов командной строки, возвращает кортеж из 4 элементов
    адрес сервера, порт, имя пользователя, пароль.
    Выполняет проверку на корректность номера порта.
    """
    parser = argparse.ArgumentParser()
    parser.add_argument("addr", default=DEFAULT_IP_ADDRESS, nargs="?")
    parser.add_argument("port", default=DEFAULT_PORT, type=int, nargs="?")
    parser.add_argument("-n", "--name", default=None, nargs="?")
    parser.add_argument("-p", "--password", default="", nargs="?")
    namespace = parser.parse_args(sys.argv[1:])
    server_address = namespace.addr
    server_port = namespace.port
    client_name = namespace.name
    client_passwd = namespace.password

    # проверим подходящий номер порта
    if not 1023 < server_port < 65536:
        logger.critical(
            f"Попытка запуска клиента с неподходящим номером порта: {server_port}. "
            f"Допустимы адреса с 1024 до 65535. Клиент завершается."
        )
        exit(1)

    return server_address, server_port, client_name, client_passwd


@log
def main():
    from gui.main_window import ClientMainWindow
    from gui.start_dialog import UserNameDialog

    # Загружаем параметы коммандной строки
    server_address, server_port, client_name, client_passwd = arg_parser()
    logger.debug("Args loaded")

    # Создаём клиентокое приложение
    client_app = QApplication(sys.argv)

    # Если имя пользователя не было указано в командной строке, то запросим его
    start_dialog = UserNameDialog()
    if not client_name or not client_passwd:
        client_app.exec_()
        # Если пользователь ввёл имя и нажал ОК, то сохраняем ведённое и
        # удаляем объект, инааче выходим
        if start_dialog.ok_pressed:
            client_name = start_dialog.client_name.text()
            client_passwd = start_dialog.client_passwd.text()
            logger.debug(f"Using USERNAME = {client_name}, PASSWD = {client_passwd}.")
        else:
            exit(0)

    # Записываем логи
    logger.info(
        f"Запущен клиент с параметрами: адрес сервера: {server_address} , порт: {server_port},"
        f" имя пользователя: {client_name}"
    )

    dir_path = Path(__file__).parent
    keys_dir = dir_path / "keys"
    key_file = keys_dir / f"{client_name}.key"

    # dir_path = os.path.dirname(os.path.realpath(__file__))
    # key_file = os.path.join(dir_path, f"{client_name}.key")
    if not os.path.exists(key_file):
        keys = RSA.generate(2048, os.urandom)
        with open(key_file, "wb") as key:
            key.write(keys.export_key())
    else:
        with open(key_file, "rb") as key:
            keys = RSA.import_key(key.read())

    # !!!keys.publickey().export_key()
    logger.debug("Keys successfully loaded.")
    # Создаём объект базы данных

    from sqlalchemy import create_engine

    engine = create_engine(
        f"sqlite:///db/client_{client_name}.db3",
        echo=False,
    )
    db = Storage(engine)

    # Создаём объект - транспорт и запускаем транспортный поток
    try:
        transport = ClientTransport(server_port, server_address, db, client_name, client_passwd, keys)
        logger.debug("Transport ready.")
    except ServerError as error:
        message = QMessageBox()
        message.critical(start_dialog, "Ошибка сервера", error.text)
        exit(1)
    transport.setDaemon(True)
    transport.start()

    # Удалим объект диалога за ненадобностью
    del start_dialog

    # Создаём GUI
    main_window = ClientMainWindow(db, transport, keys)
    main_window.make_connection(transport)
    main_window.setWindowTitle(f"Чат Программа alpha release - {client_name}")
    client_app.exec_()

    # Раз графическая оболочка закрылась, закрываем транспорт
    transport.transport_shutdown()
    transport.join()


if __name__ == "__main__":
    main()



# Errors

class IncorrectDataRecivedError(Exception):
    def __str__(self):
        return 'Принято некорректное сообщение от удалённого компьютера.'


# Исключение - ошибка сервера
class ServerError(Exception):
    def __init__(self, text):
        self.text = text

    def __str__(self):
        return self.text


# исключение - аргумент функции не словарь.
class NonDictInputError(Exception):
    def __str__(self):
        return 'Аргумент функции должен быть словарём.'


# Ошибка - отсутствует обязательное поле в принятом словаре.
class ReqFieldMissingError(Exception):
    def __init__(self, missing_field):
        self.missing_field = missing_field

    def __str__(self):
        return f'В принятом словаре отсутствует обязательное поле {self.missing_field}.'


import pytest
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from server.db_server import metadata
from server.db_server.storage import Storage


# Фикстура для настройки тестовой базы данных
@pytest.fixture(scope="module")
def test_engine():
    engine = create_engine("sqlite:///:memory:")  # Или другой URL для тестовой БД
    metadata.create_all(engine)  # Создаем структуру БД
    return engine


# Фикстура для создания сессии
@pytest.fixture(scope="function")
def session(test_engine):
    connection = test_engine.connect()
    transaction = connection.begin()
    session = sessionmaker(bind=connection)()
    yield session  # используется для тестов
    session.close()
    transaction.rollback()
    connection.close()


# Фикстура для ServerStorage
@pytest.fixture(scope="function")
def storage(session):
    return Storage(session.bind)



from sqlalchemy import create_engine, MetaData
from sqlalchemy.orm import registry

# engine = create_engine(
#     "sqlite:///db_client.db3",
#     echo=False,
#     pool_recycle=7200,
#     connect_args={"check_same_thread": False},
# )


metadata = MetaData()  # создаем экземпляр MetaData
registry = registry()  # создаем экземпляр registry



import sys
from PyQt5.QtWidgets import (
    QMainWindow,
    QAction,
    qApp,
    QApplication,
    QLabel,
    QTableView,
    QDialog,
    QPushButton,
    QLineEdit,
    QFileDialog,
    QMessageBox,
)
from PyQt5.QtGui import QStandardItemModel, QStandardItem
from PyQt5.QtCore import Qt
import os


# GUI - Создание таблицы QModel, для отображения в окне программы.
def gui_create_model(database):
    list_users = database.active_users_list()
    list_table = QStandardItemModel()
    list_table.setHorizontalHeaderLabels(
        ["Имя Клиента", "IP Адрес", "Порт", "Время подключения"]
    )
    for row in list_users:
        user, ip, port, time = row
        user = QStandardItem(user)
        user.setEditable(False)
        ip = QStandardItem(ip)
        ip.setEditable(False)
        port = QStandardItem(str(port))
        port.setEditable(False)
        # Уберём миллисекунды из строки времени, т.к. такая точность не требуется.
        time = QStandardItem(str(time.replace(microsecond=0)))
        time.setEditable(False)
        list_table.appendRow([user, ip, port, time])
    return list_table


# GUI - Функция реализующая заполнение таблицы историей сообщений.
def create_stat_model(database):
    # Список записей из базы
    hist_list = database.message_history()

    # Объект модели данных:
    list_table = QStandardItemModel()
    list_table.setHorizontalHeaderLabels(
        [
            "Имя Клиента",
            "Последний раз входил",
            "Сообщений отправлено",
            "Сообщений получено",
        ]
    )
    for row in hist_list:
        user, last_seen, sent, recvd = row
        user = QStandardItem(user)
        user.setEditable(False)
        last_seen = QStandardItem(str(last_seen.replace(microsecond=0)))
        last_seen.setEditable(False)
        sent = QStandardItem(str(sent))
        sent.setEditable(False)
        recvd = QStandardItem(str(recvd))
        recvd.setEditable(False)
        list_table.appendRow([user, last_seen, sent, recvd])
    return list_table


# Класс основного окна
class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()
        self.initUI()

    def initUI(self):
        # Кнопка выхода
        exitAction = QAction("Выход", self)
        exitAction.setShortcut("Ctrl+Q")
        exitAction.triggered.connect(qApp.quit)

        # Кнопка обновить список клиентов
        self.refresh_button = QAction("Обновить список", self)

        # Кнопка настроек сервера
        self.config_btn = QAction("Настройки сервера", self)

        # Кнопка вывести историю сообщений
        self.show_history_button = QAction("История клиентов", self)

        # Статусбар
        # dock widget
        self.statusBar()

        # Тулбар
        self.toolbar = self.addToolBar("MainBar")
        self.toolbar.addAction(exitAction)
        self.toolbar.addAction(self.refresh_button)
        self.toolbar.addAction(self.show_history_button)
        self.toolbar.addAction(self.config_btn)

        # Настройки геометрии основного окна
        # Поскольку работать с динамическими размерами мы не умеем, и мало времени на изучение, размер окна фиксирован.
        self.setFixedSize(800, 600)
        self.setWindowTitle("Messaging Server alpha release")

        # Надпись о том, что ниже список подключённых клиентов
        self.label = QLabel("Список подключённых клиентов:", self)
        self.label.setFixedSize(240, 15)
        self.label.move(10, 25)

        # Окно со списком подключённых клиентов.
        self.active_clients_table = QTableView(self)
        self.active_clients_table.move(10, 45)
        self.active_clients_table.setFixedSize(780, 400)

        # Последним параметром отображаем окно.
        self.show()


# Класс окна с историей пользователей
class HistoryWindow(QDialog):
    def __init__(self):
        super().__init__()
        self.initUI()

    def initUI(self):
        # Настройки окна:
        self.setWindowTitle("Статистика клиентов")
        self.setFixedSize(600, 700)
        self.setAttribute(Qt.WA_DeleteOnClose)

        # Кнопка закрытия окна
        self.close_button = QPushButton("Закрыть", self)
        self.close_button.move(250, 650)
        self.close_button.clicked.connect(self.close)

        # Лист с собственно историей
        self.history_table = QTableView(self)
        self.history_table.move(10, 10)
        self.history_table.setFixedSize(580, 620)

        self.show()


# Класс окна настроек
class ConfigWindow(QDialog):
    def __init__(self):
        super().__init__()
        self.initUI()

    def initUI(self):
        # Настройки окна
        self.setFixedSize(365, 260)
        self.setWindowTitle("Настройки сервера")

        # Надпись о файле базы данных:
        self.db_path_label = QLabel("Путь до файла базы данных: ", self)
        self.db_path_label.move(10, 10)
        self.db_path_label.setFixedSize(240, 15)

        # Строка с путём базы
        self.db_path = QLineEdit(self)
        self.db_path.setFixedSize(250, 20)
        self.db_path.move(10, 30)
        self.db_path.setReadOnly(True)

        # Кнопка выбора пути.
        self.db_path_select = QPushButton("Обзор...", self)
        self.db_path_select.move(275, 28)

        # Функция обработчик открытия окна выбора папки
        def open_file_dialog():
            global dialog
            dialog = QFileDialog(self)
            path = dialog.getExistingDirectory()
            path = path.replace("/", "\\")
            self.db_path.insert(path)

        self.db_path_select.clicked.connect(open_file_dialog)

        # Метка с именем поля файла базы данных
        self.db_file_label = QLabel("Имя файла базы данных: ", self)
        self.db_file_label.move(10, 68)
        self.db_file_label.setFixedSize(180, 15)

        # Поле для ввода имени файла
        self.db_file = QLineEdit(self)
        self.db_file.move(200, 66)
        self.db_file.setFixedSize(150, 20)

        # Метка с номером порта
        self.port_label = QLabel("Номер порта для соединений:", self)
        self.port_label.move(10, 108)
        self.port_label.setFixedSize(180, 15)

        # Поле для ввода номера порта
        self.port = QLineEdit(self)
        self.port.move(200, 108)
        self.port.setFixedSize(150, 20)

        # Метка с адресом для соединений
        self.ip_label = QLabel("С какого IP принимаем соединения:", self)
        self.ip_label.move(10, 148)
        self.ip_label.setFixedSize(180, 15)

        # Метка с напоминанием о пустом поле.
        self.ip_label_note = QLabel(
            " оставьте это поле пустым, чтобы\n принимать соединения с любых адресов.",
            self,
        )
        self.ip_label_note.move(10, 168)
        self.ip_label_note.setFixedSize(500, 30)

        # Поле для ввода ip
        self.ip = QLineEdit(self)
        self.ip.move(200, 148)
        self.ip.setFixedSize(150, 20)

        # Кнопка сохранения настроек
        self.save_btn = QPushButton("Сохранить", self)
        self.save_btn.move(190, 220)

        # Кнопка закрытия окна
        self.close_button = QPushButton("Закрыть", self)
        self.close_button.move(275, 220)
        self.close_button.clicked.connect(self.close)

        self.show()


if __name__ == "__main__":
    app = QApplication(sys.argv)
    main_window = MainWindow()
    main_window.statusBar().showMessage("Test Statusbar Message")
    test_list = QStandardItemModel(main_window)
    test_list.setHorizontalHeaderLabels(
        ["Имя Клиента", "IP Адрес", "Порт", "Время подключения"]
    )
    test_list.appendRow(
        [
            QStandardItem("test_user_1"),
            QStandardItem("192.198.0.5"),
            QStandardItem("23544"),
            QStandardItem("16:20:34"),
        ]
    )
    test_list.appendRow(
        [
            QStandardItem("test_user_2"),
            QStandardItem("192.198.0.8"),
            QStandardItem("33245"),
            QStandardItem("16:22:11"),
        ]
    )
    main_window.active_clients_table.setModel(test_list)
    main_window.active_clients_table.resizeColumnsToContents()
    app.exec_()

    # ----------------------------------------------------------
    # app = QApplication(sys.argv)
    # dial = ConfigWindow()

    # app.exec_()

    # ----------------------------------------------------------
    # app = QApplication(sys.argv)
    # window = HistoryWindow()
    # test_list = QStandardItemModel(window)
    # test_list.setHorizontalHeaderLabels(
    #     ["Имя Клиента", "Последний раз входил", "Отправлено", "Получено"]
    # )
    # test_list.appendRow(
    #     [
    #         QStandardItem("test1"),
    #         QStandardItem("Fri Dec 12 16:20:34 2020"),
    #         QStandardItem("2"),
    #         QStandardItem("3"),
    #     ]
    # )
    # test_list.appendRow(
    #     [
    #         QStandardItem("test2"),
    #         QStandardItem("Fri Dec 12 16:23:12 2020"),
    #         QStandardItem("8"),
    #         QStandardItem("5"),
    #     ]
    # )
    # window.history_table.setModel(test_list)
    # window.history_table.resizeColumnsToContents()

    # app.exec_()



from PyQt5.QtWidgets import (
    QDialog,
    QPushButton,
    QLineEdit,
    QApplication,
    QLabel,
    QMessageBox,
)
from PyQt5.QtCore import Qt

from db_server import Storage
from core.core import MessageProcessor


class RegisterUser(QDialog):
    """Класс диалог регистрации пользователя на сервере."""

    def __init__(self, db: Storage, server: MessageProcessor):
        super().__init__()

        self.db = db
        self.server = server

        self.setWindowTitle("Регистрация")
        self.setFixedSize(175, 183)
        self.setModal(True)
        self.setAttribute(Qt.WA_DeleteOnClose)

        self.label_username = QLabel("Введите имя пользователя:", self)
        self.label_username.move(10, 10)
        self.label_username.setFixedSize(150, 15)

        self.client_name = QLineEdit(self)
        self.client_name.setFixedSize(154, 20)
        self.client_name.move(10, 30)

        self.label_passwd = QLabel("Введите пароль:", self)
        self.label_passwd.move(10, 55)
        self.label_passwd.setFixedSize(150, 15)

        self.client_passwd = QLineEdit(self)
        self.client_passwd.setFixedSize(154, 20)
        self.client_passwd.move(10, 75)
        self.client_passwd.setEchoMode(QLineEdit.Password)
        self.label_conf = QLabel("Введите подтверждение:", self)
        self.label_conf.move(10, 100)
        self.label_conf.setFixedSize(150, 15)

        self.client_conf = QLineEdit(self)
        self.client_conf.setFixedSize(154, 20)
        self.client_conf.move(10, 120)
        self.client_conf.setEchoMode(QLineEdit.Password)

        self.btn_ok = QPushButton("Сохранить", self)
        self.btn_ok.move(10, 150)
        self.btn_ok.clicked.connect(self.save_data)

        self.btn_cancel = QPushButton("Выход", self)
        self.btn_cancel.move(90, 150)
        self.btn_cancel.clicked.connect(self.close)

        self.messages = QMessageBox()

        self.show()

    def save_data(self):
        """
        Метод проверки правильности ввода и сохранения в базу нового пользователя.
        """
        username = self.client_name.text()
        password = self.client_passwd.text()
        confirm_password = self.client_conf.text()

        if not username:
            self.messages.critical(self, "Ошибка", "Не указано имя пользователя.")
            return
        elif password != confirm_password:
            self.messages.critical(self, "Ошибка", "Введённые пароли не совпадают.")
            return
        elif self.db.check_user_existing(username):
            self.messages.critical(self, "Ошибка", "Пользователь уже существует.")
            return
        else:
            new_user = self.db.add_user(username, password)
            if not new_user:
                self.messages.critical(self, "Ошибка", "Ошибка при создании пользователя.")
                return

            # Рассылаем клиентам сообщение о необходимости обновить справочники
            self.messages.information(self, "Успех", "Пользователь успешно зарегистрирован.")
            self.server.service_update_lists()
            self.close()


if __name__ == "__main__":
    app = QApplication([])
    database = Storage()
    server = MessageProcessor("127.0.0.1", 7777, database)

    dial = RegisterUser(database, server)
    app.exec_()



from PyQt5.QtWidgets import (
    QDialog,
    QLabel,
    QLineEdit,
    QPushButton,
    QFileDialog,
    QMessageBox,
)
from PyQt5.QtCore import Qt
import os


class ConfigWindow(QDialog):
    """Класс окно настроек."""

    def __init__(self, config):
        super().__init__()
        self.config = config
        self.initUI()

    def initUI(self):
        """Настройки окна"""
        self.setFixedSize(365, 260)
        self.setWindowTitle("Настройки сервера")
        self.setAttribute(Qt.WA_DeleteOnClose)
        self.setModal(True)

        # Надпись о файле базы данных:
        self.db_path_label = QLabel("Путь до файла базы данных: ", self)
        self.db_path_label.move(10, 10)
        self.db_path_label.setFixedSize(240, 15)

        # Строка с путём базы
        self.db_path = QLineEdit(self)
        self.db_path.setFixedSize(250, 20)
        self.db_path.move(10, 30)
        self.db_path.setReadOnly(True)

        # Кнопка выбора пути.
        self.db_path_select = QPushButton("Обзор...", self)
        self.db_path_select.move(275, 28)

        # Метка с именем поля файла базы данных
        self.db_file_label = QLabel("Имя файла базы данных: ", self)
        self.db_file_label.move(10, 68)
        self.db_file_label.setFixedSize(180, 15)

        # Поле для ввода имени файла
        self.db_file = QLineEdit(self)
        self.db_file.move(200, 66)
        self.db_file.setFixedSize(150, 20)

        # Метка с номером порта
        self.port_label = QLabel("Номер порта для соединений:", self)
        self.port_label.move(10, 108)
        self.port_label.setFixedSize(180, 15)

        # Поле для ввода номера порта
        self.port = QLineEdit(self)
        self.port.move(200, 108)
        self.port.setFixedSize(150, 20)

        # Метка с адресом для соединений
        self.ip_label = QLabel("С какого IP принимаем соединения:", self)
        self.ip_label.move(10, 148)
        self.ip_label.setFixedSize(180, 15)

        # Метка с напоминанием о пустом поле.
        self.ip_label_note = QLabel(
            " оставьте это поле пустым, чтобы\n принимать соединения с любых адресов.",
            self,
        )
        self.ip_label_note.move(10, 168)
        self.ip_label_note.setFixedSize(500, 30)

        # Поле для ввода ip
        self.ip = QLineEdit(self)
        self.ip.move(200, 148)
        self.ip.setFixedSize(150, 20)

        # Кнопка сохранения настроек
        self.save_btn = QPushButton("Сохранить", self)
        self.save_btn.move(190, 220)

        # Кнапка закрытия окна
        self.close_button = QPushButton("Закрыть", self)
        self.close_button.move(275, 220)
        self.close_button.clicked.connect(self.close)

        self.db_path_select.clicked.connect(self.open_file_dialog)

        self.show()

        self.db_path.insert(self.config["SETTINGS"]["Database_path"])
        self.db_file.insert(self.config["SETTINGS"]["Database_file"])
        self.port.insert(self.config["SETTINGS"]["Default_port"])
        self.ip.insert(self.config["SETTINGS"]["Listen_Address"])
        self.save_btn.clicked.connect(self.save_server_config)

    def open_file_dialog(self):
        """Метод обработчик открытия окна выбора папки."""
        global dialog
        dialog = QFileDialog(self)
        path = dialog.getExistingDirectory()
        path = path.replace("/", "\\")
        self.db_path.clear()
        self.db_path.insert(path)

    def save_server_config(self):
        """
        Метод сохранения настроек.
        Проверяет правильность введённых данных и
        если всё правильно сохраняет ini файл.
        """
        global config_window
        message = QMessageBox()
        self.config["SETTINGS"]["Database_path"] = self.db_path.text()
        self.config["SETTINGS"]["Database_file"] = self.db_file.text()
        try:
            port = int(self.port.text())
        except ValueError:
            message.warning(self, "Ошибка", "Порт должен быть числом")
        else:
            self.config["SETTINGS"]["Listen_Address"] = self.ip.text()
            if 1023 < port < 65536:
                self.config["SETTINGS"]["Default_port"] = str(port)
                dir_path = os.path.dirname(os.path.realpath(__file__))
                dir_path = os.path.join(dir_path, "..")
                with open(f"{dir_path}/{'server.ini'}", "w") as conf:
                    self.config.write(conf)
                    message.information(self, "OK", "Настройки успешно сохранены!")
            else:
                message.warning(self, "Ошибка", "Порт должен быть от 1024 до 65536")



from sqlalchemy.orm import sessionmaker
import datetime

from db_client.database import metadata
from db_client.models import MessageHistory, Contacts, KnownUsers


class Storage:
    def __init__(self, engine):
        metadata.create_all(engine)  # Инициализация базы данных и создание таблиц
        self.session = sessionmaker(bind=engine)()  # Создание сессии

    # Функция добавления контактов
    def add_contact(self, contact):
        if not self.session.query(Contacts).filter_by(name=contact).count():
            contact_row = Contacts(contact)
            self.session.add(contact_row)
            self.session.commit()

    # Функция удаления контакта
    def del_contact(self, contact):
        self.session.query(Contacts).filter_by(name=contact).delete()
        self.session.commit()

    # Функция добавления известных пользователей.
    # Пользователи получаются только с сервера, поэтому таблица очищается.
    def add_users(self, users_list):
        self.session.query(KnownUsers).delete()
        for user in users_list:
            user_row = KnownUsers(user)
            self.session.add(user_row)
        self.session.commit()

    # Функция сохраняет сообщения
    def save_message(self, from_user, to_user, message):
        message_row = MessageHistory(from_user, to_user, message)
        self.session.add(message_row)
        self.session.commit()

    # Функция возвращает контакты
    def get_contacts(self):
        return [contact[0] for contact in self.session.query(Contacts.name).all()]

    # Функция возвращает список известных пользователей
    def get_users(self):
        return [user[0] for user in self.session.query(KnownUsers.username).all()]

    # Функция проверяет наличие пользователя в таблице Известных Пользователей
    def check_user(self, user):
        if self.session.query(KnownUsers).filter_by(username=user).count():
            return True
        else:
            return False

    # Функция проверяет наличие пользователя в таблице Контактов
    def check_contact(self, contact):
        if self.session.query(Contacts).filter_by(name=contact).count():
            return True
        else:
            return False

    def contacts_clear(self):
        """Метод, очищающий таблицу со списком контактов."""
        self.session.query(Contacts).delete()
        self.session.commit()

    # Функция возвращает историю переписки
    def get_history(self, from_who=None, to_who=None):
        query = self.session.query(MessageHistory)
        if from_who:
            query = query.filter_by(from_user=from_who)
        if to_who:
            query = query.filter_by(to_user=to_who)
        return [
            (
                history_row.from_user,
                history_row.to_user,
                history_row.message,
                history_row.date,
            )
            for history_row in query.all()
        ]


# отладка
if __name__ == "__main__":
    from sqlalchemy import create_engine

    engine = create_engine(
        "sqlite:///db_client.db3",
        echo=False,
        pool_recycle=7200,
        connect_args={"check_same_thread": False},
    )
    test_db = Storage(engine)

    for i in ["test3", "test4", "test5"]:
        test_db.add_contact(i)
    test_db.add_contact("test4")
    test_db.add_users(["test1", "test2", "test3", "test4", "test5"])
    test_db.save_message("test1", "test2", f"Привет! я тестовое сообщение от {datetime.datetime.now()}!")
    test_db.save_message(
        "test2",
        "test1",
        f"Привет! я другое тестовое сообщение от {datetime.datetime.now()}!",
    )
    print(test_db.get_contacts())
    print(test_db.get_users())
    print(test_db.check_user("test1"))
    print(test_db.check_user("test10"))
    print(test_db.get_history("test2"))
    print(test_db.get_history(to_who="test2"))
    print(test_db.get_history("test3"))
    test_db.del_contact("test4")
    print(test_db.get_contacts())
